USE [common_db]
GO

/*
	EXISTS functions
*/
CREATE PROCEDURE [dbo].[CheckObjectExists] 
	@name NVARCHAR(255), 
	@xtype NVARCHAR(3), 
	@exists BIT OUTPUT
AS
BEGIN	
	IF EXISTS (SELECT [name] FROM [dbo].[sysobjects] WHERE [name] = @name AND [xtype] = @xtype)
		SET @exists = 1
	ELSE
		SET @exists = 0
	RETURN 0
END
GO

CREATE PROCEDURE [dbo].[DropObjectIfExists]
	@name NVARCHAR(255),
	@xtype NVARCHAR(3)
AS
BEGIN
	DECLARE @exists BIT
	EXECUTE [dbo].[CheckObjectExists] @name, @xtype, @exists OUTPUT	
	IF @exists = 1
	BEGIN
		DECLARE @obj_type VARCHAR(32)
		IF @xtype = 'U' SET @obj_type = 'TABLE'
		ELSE IF @xtype = 'P' SET @obj_type = 'PROCEDURE'
		ELSE IF @xtype = 'TR' SET @obj_type = 'TRIGGER'
		ELSE IF @xtype = 'FN' SET @obj_type = 'FUNCTION'
		ELSE IF @xtype = 'IF' SET @obj_type = 'FUNCTION'
		DECLARE @obj_name VARCHAR(255)
		SET @obj_name = '[dbo].[' + @name + ']'
		DECLARE @statement NVARCHAR(1000) 
		SET @statement = N'DROP ' + @obj_type + N' ' + @obj_name
		EXECUTE [dbo].[sp_executesql] @statement		
	END
	RETURN 0
END
GO

CREATE PROCEDURE [dbo].[CheckTableExists] 
	@name NVARCHAR(255), 
	@exists BIT OUTPUT
AS
BEGIN	
	EXECUTE [dbo].[CheckObjectExists] @name, 'U', @exists OUTPUT
END
GO

CREATE PROCEDURE [dbo].[CheckProcedureExists]
	@name NVARCHAR(255), 
	@exists BIT OUTPUT
AS
BEGIN	
	EXECUTE [dbo].[CheckObjectExists] @name, 'P', @exists OUTPUT
END
GO

CREATE PROCEDURE [dbo].[CheckTriggerExists] 
	@name NVARCHAR(255), 
	@exists BIT OUTPUT
AS
BEGIN	
	EXECUTE [dbo].[CheckObjectExists] @name, 'TR', @exists OUTPUT
END
GO

CREATE PROCEDURE [dbo].[CheckFunctionExists] 
	@name NVARCHAR(255), 
	@exists BIT OUTPUT
AS
BEGIN	
	EXECUTE [dbo].[CheckObjectExists] @name, 'FN', @exists OUTPUT
END
GO

CREATE FUNCTION [dbo].[f_CheckObjectExists] (
	@name NVARCHAR(255), 
	@xtype NVARCHAR(3)
	)
RETURNS BIT
AS
BEGIN
	IF EXISTS (SELECT [name] FROM [dbo].[sysobjects] WHERE [name] = @name AND [xtype] = @xtype)
		RETURN 1
	ELSE
		RETURN 0
	RETURN 0	
END
GO

CREATE FUNCTION [dbo].[f_CheckTableExists] (
	@name NVARCHAR(255)
	)
RETURNS BIT
AS
BEGIN
	RETURN [dbo].[f_CheckObjectExists](@name, 'U')
END
GO

CREATE FUNCTION [dbo].[f_CheckProcedureExists] (
	@name NVARCHAR(255)	
	)
RETURNS BIT
AS
BEGIN
	RETURN [dbo].[f_CheckObjectExists](@name, 'P')
END
GO

CREATE FUNCTION [dbo].[f_CheckTriggerExists] (
	@name NVARCHAR(255)
	)
RETURNS BIT
AS
BEGIN
	RETURN [dbo].[f_CheckObjectExists](@name, 'TR')
END
GO

CREATE FUNCTION [dbo].[f_CheckFunctionExists] (
	@name NVARCHAR(255)
	)
RETURNS BIT
AS
BEGIN
	RETURN [dbo].[f_CheckObjectExists](@name, 'FN')
END
GO
/*end*/

