USE [common_db]
GO

/*
	[objects] table
	store's global references(id's) for all objects in database (tables, rows, etc.)
	all database objects must have references for this table
	[id_table], [id_row] presents identifiers for table of objects and row in this table	
*/

EXECUTE [dbo].[DropObjectIfExists] 'objects', 'U'
GO
CREATE TABLE [dbo].[objects] (
	[id]			BIGINT IDENTITY PRIMARY KEY CLUSTERED,	
	[id_table]		BIGINT NOT NULL DEFAULT 0,
	[id_row]		BIGINT NOT NULL DEFAULT 0,
	[description]	NTEXT DEFAULT NULL
)
GO

INSERT INTO [dbo].[objects] ([description]) VALUES ('global root object');
GO
/*end*/

/*
	[objects_relations] table
	presents global relations (not foreign keys!!!) of database objects.
	use for creating tree structure of objects
*/
EXECUTE [dbo].[DropObjectIfExists] 'objects_relations', 'U'
GO
CREATE TABLE [dbo].[objects_relations] (
	[id_relation]	BIGINT IDENTITY PRIMARY KEY CLUSTERED,
	[id_object]		BIGINT NOT NULL,
	[id_child]		BIGINT NOT NULL,
	CONSTRAINT [fk_objects_relations_id_object] 
		FOREIGN KEY ([id_object]) 
		REFERENCES [objects] ([id]),
	CONSTRAINT [fk_objects_relations_id_child]
		FOREIGN KEY ([id_child])
		REFERENCES [objects] ([id])
)
GO
/*end*/

/*
	GetParentsCount - get's the count of parents for some object	
*/
EXECUTE [dbo].[DropObjectIfExists] 'GetParentsCount', 'P'
GO
CREATE PROCEDURE [dbo].[GetParentsCount] 
	@id_child BIGINT, 
	@parents_cnt INT OUTPUT
AS
BEGIN
	SELECT @parents_cnt = count(*) FROM [dbo].[objects_relations] WHERE [id_child] = @id_child	
	RETURN 0
END
GO
/*end*/

/*
	GetChildsCount - get's the count of childs for some object	
*/
EXECUTE [dbo].[DropObjectIfExists] 'GetChildsCount', 'P'
GO
CREATE PROCEDURE [dbo].[GetChildsCount] 
	@id_parent BIGINT, 
	@childs_cnt INT OUTPUT
AS 
BEGIN
	SELECT @childs_cnt = count(*) FROM [dbo].[objects_relations] WHERE [id_object] = @id_parent
	RETURN 0
END
GO
/*end*/

/*
	GetParents - get's all of the parents for some object
*/
EXECUTE [dbo].[DropObjectIfExists] 'GetParents', 'P'
GO
CREATE PROCEDURE [dbo].[GetParents] 
	@id_child BIGINT
AS
BEGIN
	SELECT [id_object] FROM [dbo].[objects_relations] WHERE [id_child] = @id_child
	RETURN 0
END
GO
/*end*/

/*
	GetParents - get's all of the childs for some object
*/
EXECUTE [dbo].[DropObjectIfExists] 'GetChilds', 'P'
GO
CREATE PROCEDURE [GetChilds] 
	@id_parent BIGINT
AS
BEGIN
	SELECT [id_child] FROM [dbo].[objects_relations] WHERE [id_object] = @id_parent
	RETURN 0
END
GO
/*end*/