USE [common_db]
GO

EXECUTE [dbo].[DropObjectIfExists] 'f_GetAllTables', 'IF'
GO
CREATE FUNCTION [dbo].[f_GetAllTables] (
	)
RETURNS TABLE
AS
RETURN (SELECT * FROM [dbo].[tables])
GO

EXECUTE [dbo].[DropObjectIfExists] 'f_GetEditableTables', 'IF'
GO
CREATE FUNCTION [dbo].[f_GetEditableTables] (
	)
RETURNS TABLE
AS
RETURN (SELECT * FROM [dbo].[tables] WHERE [is_editable] = 1)
GO

EXECUTE [dbo].[DropObjectIfExists] 'f_GetSystemTables', 'IF'
GO
CREATE FUNCTION [dbo].[f_GetSystemTables] (
	)
RETURNS TABLE
AS
RETURN (SELECT * FROM [dbo].[tables] WHERE [is_system] = 1)
GO

EXECUTE [dbo].[DropObjectIfExists] 'GetRows', 'P'
GO
CREATE PROCEDURE [dbo].[GetRows]
	@table NVARCHAR(255),
	@fields NVARCHAR(255) = N'*',
	@filters NVARCHAR(255) = NULL
AS
BEGIN
	DECLARE @statement NVARCHAR(2000)
	SET @statement = N'SELECT ' + @fields + N' FROM [dbo].[' + @table + N']'
	IF @filters IS NOT NULL
		SET @statement = @statement + N' WHERE ' + @filters
	EXECUTE [dbo].[sp_executesql] @statement
END
GO

EXECUTE [dbo].[DropObjectIfExists] 'AddStandartField', 'P'
GO
CREATE PROCEDURE [dbo].[AddStandartField]
	@table_name NVARCHAR(255),
	@type_cat NVARCHAR(255),
	@type_name NVARCHAR(255),
	@field_name NVARCHAR(255),	
	@size TINYINT = NULL,
	@descr NVARCHAR(2000) = NULL,
	@def_val SQL_VARIANT = NULL,
	@is_not_null BIT = 0
AS
BEGIN
	DECLARE @table_id BIGINT
	SELECT @table_id = [dbo].[f_GetTableId](@table_name)	
	DECLARE @cat_id BIGINT
	SELECT @cat_id = [dbo].[f_GetDataTypeCategoryId](@type_cat)	
	DECLARE @type_id BIGINT
	SELECT @type_id = [dbo].[f_GetDataTypeId](@type_name)		
	INSERT INTO [dbo].[tables_fields] 
	([title],[id_table],[id_data_type_category],[id_data_type],[size],[description],[default_value],[is_not_null])
	VALUES
	(@field_name,@table_id,@cat_id,@type_id,@size,@descr,@def_val,@is_not_null)
	RETURN 0
END
GO	

EXECUTE [dbo].[DropObjectIfExists] 'RemoveField', 'P'
GO
CREATE PROCEDURE [dbo].[RemoveField]
	@table_name NVARCHAR(255),
	@field_name NVARCHAR(255)
AS
BEGIN
	DECLARE @table_id BIGINT
	SELECT @table_id = [dbo].[f_GetTableId] (@table_name)
	DECLARE @field_id BIGINT
	DECLARE @field_type NVARCHAR(255)
	DECLARE @ref_to_table BIGINT
	DECLARE @ref_by_table NVARCHAR(255)
	SELECT 
		@field_id = [tf].[id], 
		@field_type = [dt].[title], 
		@ref_to_table = [tf].[reference_to_id_table],
		@ref_by_table = [tf].[reference_by_table]
		FROM [dbo].[tables_fields] [tf], [dbo].[data_types] [dt]
		WHERE 
		[tf].[title] = @field_name AND 
		[tf].[id_table] = @table_id AND 
		[tf].[id_data_type] = [dt].[id]
	DELETE FROM [dbo].[tables_fields] WHERE [id] = @field_id
	IF @field_type = '1-m'
	BEGIN
		EXECUTE [dbo].[DropObjectIfExists] @ref_by_table, 'U'
		DECLARE @tab NVARCHAR(255)
		SELECT @tab = [dbo].[f_GetTableName] (@ref_to_table)
		EXECUTE [dbo].[DropObjectIfExists] @ref_to_table, @field_name
	END
END
GO

EXECUTE [dbo].[DropObjectIfExists] 'CreateTable', 'P'
GO
CREATE PROCEDURE [dbo].[CreateTable]
	@table_name NVARCHAR(255),
	@table_descr NVARCHAR(2000) = NULL,
	@has_descr BIT = 1,
	@is_editable BIT = 1,
	@is_system BIT = 1
AS 
BEGIN
	IF @table_descr IS NULL
		SET @table_descr = N'[' + @table_name + N'] table'
	INSERT INTO [dbo].[tables] 
		([title],[description],[is_editable],[is_system]) 
		VALUES
		(@table_name, @table_descr, @is_editable, @is_system)
	DECLARE @ins_id BIGINT
	SELECT @ins_id = [dbo].[f_GetTableId](@table_name)
	DECLARE @descr NVARCHAR(300)
	SET @descr = N'Description of [' + @table_name + N']'
	IF @has_descr = 1
		EXECUTE [dbo].[AddStandartField]
			@table_name,
			N'unicode_strings',
			N'NTEXT',
			N'description',
			NULL,
			@descr
	RETURN 0			
END
GO

EXECUTE [dbo].[CreateTable] N'TEST'
GO

